@@ -1,24 +1,28 @@
 package Language::Expr::Compiler::JS;
 BEGIN {
-  $Language::Expr::Compiler::JS::VERSION = '0.05';
+  $Language::Expr::Compiler::JS::VERSION = '0.06';
 }
 # Compile Language::Expr expression to JS
 
+use 5.010;
 use Any::Moose;
 with 'Language::Expr::EvaluatorRole';
 extends 'Language::Expr::Evaluator';
 
 use UUID::Tiny ':std';
+use Language::Expr::Interpreter::Default;
 
 
 has todo => (is => 'rw', default => sub { [] });
 
 
+has func_mapping => (is => 'rw', default => sub { {} });
+
 
 sub rule_pair {
     my ($self, %args) = @_;
     my $match = $args{match};
-    "$match->{key} => $match->{value}";
+    "$match->{key}:$match->{value}";
 }
 
 sub rule_or_xor {
@@ -30,9 +34,12 @@ sub rule_or_xor {
         my $op = shift @{$match->{op}//=[]};
         last unless $op;
         if    ($op eq '||') { push @res, " || $term" }
-        elsif ($op eq '//') { push @res, " // $term" }
-        # add parenthesis because perl's xor precendence is low
-        elsif ($op eq '^^') { @res = ("(", @res, " xor $term)") }
+        elsif ($op eq '//') { @res = ("(function() { let _x = (",
+                                      @res, "); return _x==null ? (",
+                                      $term, ") : _x })()") }
+        elsif ($op eq '//') { @res = ("(function() { let _a = (",
+                                      @res, "); let _b = ($term); ",
+                                      "return _a&&!_b || !_a&&_b ? _a : _b })()") }
     }
     join "", grep {defined} @res;
 }
@@ -85,12 +92,29 @@ sub rule_comparison3 {
     for my $term (@{$match->{operand}}) {
         my $op = shift @{$match->{op}//=[]};
         last unless $op;
-        if    ($op eq '<=>') { push @res, " <=> $term" }
-        elsif ($op eq 'cmp') { push @res, " cmp $term" }
+        if    ($op eq '<=>') { @res = ("(function() { let _a = (",
+                                      @res, "); let _b = ($term); ",
+                                      "return _a > _b ? 1 : (_a < _b ? -1 : 0) })()") }
+        elsif ($op eq 'cmp') { @res = ("(function() { let _a = (",
+                                      @res, ") + ''; let _b = ($term) + ''; ",
+                                      "return _a > _b ? 1 : (_a < _b ? -1 : 0) })()") }
     }
     join "", grep {defined} @res;
 }
 
+sub _comparison1 {
+    my ($opd1, $op, $opd2) = @_;
+    given ($op) {
+        when ('eq') { return "(function() { let _a = ($opd1) + ''; let _b = ($opd2) + ''; return $opd1 == $opd2 })()" }
+        when ('ne') { return "(function() { let _a = ($opd1) + ''; let _b = ($opd2) + ''; return $opd1 != $opd2 })()" }
+        when ('lt') { return "(function() { let _a = ($opd1) + ''; let _b = ($opd2) + ''; return $opd1 <  $opd2 })()" }
+        when ('le') { return "(function() { let _a = ($opd1) + ''; let _b = ($opd2) + ''; return $opd1 <= $opd2 })()" }
+        when ('gt') { return "(function() { let _a = ($opd1) + ''; let _b = ($opd2) + ''; return $opd1 >  $opd2 })()" }
+        when ('ge') { return "(function() { let _a = ($opd1) + ''; let _b = ($opd2) + ''; return $opd1 >= $opd2 })()" }
+        default { return "($opd1 $op $opd2)" }
+    }
+}
+
 sub rule_comparison {
     my ($self, %args) = @_;
     my $match = $args{match};
@@ -106,7 +130,6 @@ sub rule_comparison {
         elsif ($op eq '!=' ) { push @ops, '!='  }
         elsif ($op eq 'eq' ) { push @ops, 'eq'  }
         elsif ($op eq 'ne' ) { push @ops, 'ne'  }
-        elsif ($op eq 'ne' ) { push @ops, 'ne'  }
         elsif ($op eq '<'  ) { push @ops, '<'   }
         elsif ($op eq '<=' ) { push @ops, '<='  }
         elsif ($op eq '>'  ) { push @ops, '>'   }
@@ -130,9 +153,9 @@ sub rule_comparison {
             $opd1 = pop @opds;
         }
         if (@res) {
-            @res = ("(($opd1 $op $opd2) ? ", @res, " : '')");
+            @res = ("("._comparison1($opd1, $op, $opd2)." ? ", @res, " : false)");
         } else {
-            push @res, "($opd1 $op $opd2)";
+            push @res, _comparison1($opd1, $op, $opd2);
         }
         $lastopd = $opd1;
     }
@@ -161,7 +184,7 @@ sub rule_add {
     for my $term (@{$match->{operand}}) {
         my $op = shift @{$match->{op}//=[]};
         last unless $op;
-        if    ($op eq '.') { push @res, " . $term" }
+        if    ($op eq '.') { @res = ("'' + ", @res, " + $term") }
         if    ($op eq '+') { push @res, " + $term" }
         if    ($op eq '-') { push @res, " - $term" }
     }
@@ -179,7 +202,7 @@ sub rule_mult {
         if    ($op eq '*') { push @res, " * $term" }
         if    ($op eq '/') { push @res, " / $term" }
         if    ($op eq '%') { push @res, " % $term" }
-        if    ($op eq 'x') { push @res, " x $term" }
+        if    ($op eq 'x') { @res = ("(new Array(1 + $term).join(", @res, "))") }
     }
     join "", grep {defined} @res;
 }
@@ -203,9 +226,9 @@ sub rule_power {
     my ($self, %args) = @_;
     my $match = $args{match};
     my @res;
-    push @res, shift @{$match->{operand}};
-    for my $term (@{$match->{operand}}) {
-        push @res, " ** $term";
+    push @res, pop @{$match->{operand}};
+    for my $term (reverse @{$match->{operand}}) {
+        @res = ("Math.pow($term, ", @res, ")");
     }
     join "", grep {defined} @res;
 }
@@ -219,10 +242,7 @@ sub rule_subscripting {
     my $res;
     for my $s (@ss) {
         $opd = $res if defined($res);
-        $res = qq!(do { my (\$v) = ($opd); my (\$s) = ($s); !.
-            qq!if (ref(\$v) eq 'HASH') { \$v->{\$s} } !.
-                qq!elsif (ref(\$v) eq 'ARRAY') { \$v->[\$s] } else { !.
-                    qq!die "Invalid subscript \$s for \$v" } })!;
+        $res = $opd . "[$s]";
     }
     $res;
 }
@@ -240,7 +260,7 @@ sub rule_hash {
 }
 
 sub rule_undef {
-    "undef";
+    "null";
 }
 
 sub rule_squotestr {
@@ -254,29 +274,36 @@ sub rule_dquotestr {
 sub rule_bool {
     my ($self, %args) = @_;
     my $match = $args{match};
-    if ($match->{bool} eq 'true') { 1 } else { "''" }
+    if ($match->{bool} eq 'true') { "true" } else { "false" }
 }
 
 sub rule_num {
     my ($self, %args) = @_;
     my $match = $args{match};
-    if    ($match->{num} eq 'inf') { '"Inf"' }
-    elsif ($match->{num} eq 'nan') { '"NaN"' }
+    if    ($match->{num} eq 'inf') { 'Infinity' }
+    elsif ($match->{num} eq 'nan') { 'NaN' }
     else                           { $match->{num}+0 }
 }
 
 sub rule_var {
     my ($self, %args) = @_;
     my $match = $args{match};
-    "\$$match->{var}";
+    "$match->{var}";
 }
 
 sub rule_func {
     my ($self, %args) = @_;
     my $match = $args{match};
     my $f = $match->{func_name};
+    my $fmap = $self->func_mapping->{$f};
+    $f = $fmap if $fmap;
     my $args = $match->{args};
-    "$f(".join(", ", @$args).")";
+    if (substr($f, 0, 1) eq '.') {
+        my $invoc = shift @$args;
+        return "($invoc)$f(".join(", ", @$args).")";
+    } else {
+        return "$f(".join(", ", @$args).")";
+    }
 }
 
 sub _map_grep_usort {
@@ -285,10 +312,15 @@ sub _map_grep_usort {
     my $ary = $match->{array};
     my $expr = $match->{expr};
 
-    my $perlop = $which eq 'map' ? 'map' : $which eq 'grep' ? 'grep' : 'sort';
     my $todoid = __uuidgen(); # yes, this is not proper
     push @{ $self->todo }, [$todoid, $expr];
-    "[$perlop({ TODO-$todoid } \@{$ary})]";
+    if ($which eq 'map') {
+        return "($ary).map(function(_){ return (TODO-$todoid); })";
+    } elsif ($which eq 'grep') {
+        return "($ary).filter(function(_){ return (TODO-$todoid); })";
+    } elsif ($which eq 'usort') {
+        return "($ary).map(function(x) x).sort(function(a, b){ return (TODO-$todoid); })";
+    }
 }
 
 sub rule_func_map {
@@ -325,10 +357,9 @@ sub __quote {
         my $o = ord($c);
         if    ($c eq '"') { push @c, '\\"' }
         elsif ($c eq "\\") { push @c, "\\\\" }
-        elsif ($c eq '$') { push @c, "\\\$" }
-        elsif ($c eq '@') { push @c, '\\@' }
         elsif ($o >= 32 && $o <= 127) { push @c, $c }
-        else { push @c, sprintf("\\x%02x", $o) }
+        elsif ($o > 255) { push @c, sprintf("\\x{%04x}", $o) }
+        else  { push @c, sprintf("\\x%02x", $o) }
     }
     '"' . join("", @c) . '"';
 }
@@ -338,7 +369,7 @@ sub __uuidgen {
 }
 
 
-sub perl {
+sub js {
     my ($self, $expr) = @_;
     my $res = Language::Expr::Parser::parse_expr($expr, $self);
     for my $todo (@{ $self->todo }) {
@@ -353,7 +384,7 @@ sub perl {
 
 sub eval {
     my ($self, $expr) = @_;
-    my $res = eval $self->perl($expr);
+    my $res = eval $self->js($expr);
     die $@ if $@;
     $res;
 }
@@ -371,7 +402,7 @@ Language::Expr::Compiler::JS
 
 =head1 VERSION
 
-version 0.05
+version 0.06
 
 =head1 DESCRIPTION
 
@@ -379,6 +410,23 @@ Compiles Language::Expr expression to JS code. Some notes:
 
 =over 4
 
+=item * JavaScript version
+
+This compiler emits JavaScript 1.8.1 code (it uses several Array
+methods like map() and filter() [supported since JavaScript 1.6],
+'let' lexical variables [supported since JavaScript 1.7 / Firefox 2]
+and native JSON [supported since JavaScript 1.8.1 / Firefox 3.5]).
+
+Currently to test emitted JavaScript code, we use Spidermonkey 1.9.1+
+JavaScript interpreter (typically located in /usr/bin/js) as the
+L<JavaScript> and L<JE> modules are still not up to par.
+
+=item * JavaScript-ness
+
+The emitted JS code will follow JavaScript's weak typing and coercion
+rules, e.g. Expr '1+"2"' will simply be translated to JavaScript
+'1+"2"' and will result in "12".
+
 =item * Currently strings are rudimentary escaped.
 
 Data dumping modules can't be used currently due to segfaults (at
@@ -386,13 +434,20 @@ least in 5.10.1).
 
 =item * Variables by default simply use JavaScript variables.
 
-E.g. $a becomes $a, and so on. Be careful not to make variables which
+E.g. $a becomes a, and so on. Be careful not to make variables which
 are invalid in JavaScript, e.g. $.. or ${foo/bar}.
 
 You can subclass and override rule_var() if you want to provide your
 own variables.
 
-=item * Functions by default simply use Perl functions.
+=item * Functions by default simply use Javascript functions.
+
+foo() becomes foo(). Except those mentioned in B<func_mapping>
+(e.g. rand() becomes Math.rand() if func_mapping->{rand} is
+'Math.rand').
+
+Or you can subclass and override rule_func() for more translation
+freedom.
 
 =back
 
@@ -402,14 +457,22 @@ own variables.
 
 Used to remember which subexpression need to be parsed later.
 
+=head2 func_mapping => HASHREF
+
+Mapping from Expr function to JavaScript functions/methods. Example:
+
+ { ceil => 'Math.ceil',
+   uc   => '.toUpperCase',
+ }
+
 =head1 METHODS
 
 =for Pod::Coverage ^(rule|expr)_.+
 
-=head2 perl($expr) => $perl_code
+=head2 js($expr) => $js_code
 
-Convert Language::Expr expression into Perl code. Dies if there is
-syntax error in expression.
+Convert Language::Expr expression into JavaScript code. Dies if there
+is syntax error in expression.
 
 =head1 AUTHOR
 
